# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.part.page_info import (
    PageInfo
)


class YoutubeListResponse(BaseModel):
    model_config = get_base_model_config()

    kind: str | None = Field(default=None)
    etag: str | None = Field(default=None)
    next_page_token: str | None = Field(default=None)
    prev_page_token: str | None = Field(default=None)
    page_info: PageInfo | None = Field(default=None)
    items: list[any | None] = Field(default=[])



# Pydantic
from pydantic import ConfigDict
from pydantic.alias_generators import to_camel

def get_base_model_config() -> ConfigDict:
    return ConfigDict(
        arbitrary_types_allowed = True,
        alias_generator = to_camel,
        extra = "forbid"
    )



import youtube_pydantic_models.models.responses
import youtube_pydantic_models.models.resources
import youtube_pydantic_models.models.part
import youtube_pydantic_models.models.subpart
import youtube_pydantic_models.models.base_model_config
import youtube_pydantic_models.models.youtube_client



import unittest
import json


class TestYoutubeResource(unittest.TestCase):
    def init_params(
        self,
        data_path: str,
        model
    ) -> tuple:
        data = self.get_json_data(data_path)
        if not data:
            raise Exception("example data not found")

        json_data = data["items"][0]
        model_data = model(
            **json_data
        ).model_dump(
            by_alias = True,
            exclude_none = True
        )

        return json_data, model_data

    def assert_equal_model_parts(self, part: str) -> None:
        json_part = self.json_data[part]
        model_part = self.model_data[part]
        print(json_part)
        print()
        print(model_part)
        if type(json_part) == dict and type(model_part) == dict:
            assert self.is_included_dict(json_part, model_part)
        else:
            assert json_part == model_part
    
    def get_json_data(self, file_name: str) -> dict | None:
        with open(
            f"youtube_pymodel/tests/data/{file_name}",
            "r"
        ) as data:
            return json.loads(data.read())
        return None

    def is_included_dict(
        self,
        inner_dict: dict,
        outer_dict: dict
    ) -> bool:
        try:
            for key, value in inner_dict.items():
                if type(value) == dict:
                    self.is_included_dict(value, outer_dict[key])
                if value != outer_dict[key]:
                    return False
            return True
        except:
            return False



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class BaseTopicDetails(BaseModel):
    model_config = get_base_model_config()

    topic_ids: list[str] | None = Field(default=None)
    topic_categories: list[str] | None = Field(default=None)


class VideoTopicDetails(BaseTopicDetails):
    model_config = get_base_model_config()

    relevant_topic_ids: list[str] | None = Field(default=None)



# Python
from datetime import datetime
# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class BaseStatus(BaseModel):
    model_config = get_base_model_config()

    privacy_status: str | None = Field(default=None)


class SharedStatus(BaseStatus):
    model_config = get_base_model_config()

    made_for_kids: bool | None = Field(default=None)
    self_declared_made_for_kids: bool | None = Field(default=None)


class ChannelStatus(SharedStatus):
    model_config = get_base_model_config()

    is_linked: bool | None = Field(default=None)
    long_uploads_status: str | None = Field(default=None)


class VideoStatus(SharedStatus):
    model_config = get_base_model_config()

    upload_status: str | None = Field(default=None)
    failure_reason: str | None = Field(default=None)
    rejection_reason: str | None = Field(default=None)
    publish_at: datetime | str | None = Field(default=None)
    license: str | None = Field(default=None)
    embeddable: bool | None = Field(default=None)
    public_stats_viewable: bool | None = Field(default=None)



from models.resources.playlist_resource import YoutubePlaylistResource
from tests.test_youtube_resource import TestYoutubeResource


class TestPlaylistResource(TestYoutubeResource):
    def setUp(self):
        params = self.init_params(
            "playlist_example_data.json",
            YoutubePlaylistResource
        )
        self.json_data = params[0]
        self.model_data = params[1]
    
    def test_equal_kind_part(self):
        self.assert_equal_model_parts("kind")
    
    def test_equal_etag_part(self):
        self.assert_equal_model_parts("etag")
    
    def test_equal_id_part(self):
        self.assert_equal_model_parts("id")

    def test_equal_snippet_part(self):
        self.assert_equal_model_parts("snippet")
    
    def test_equal_status_part(self):
        self.assert_equal_model_parts("status")
    
    def test_equal_content_details_part(self):
        self.assert_equal_model_parts("contentDetails")
    
    def test_equal_player_part(self):
        self.assert_equal_model_parts("player")



# Pydantic
from pydantic import Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.resources.base_resource import (
    YoutubeBaseResource
)
from youtube_pydantic_models.models.part.snippet import (
    VideoSnippet
)
from youtube_pydantic_models.models.part.content_details import (
    VideoContentDetails
)
from youtube_pydantic_models.models.part.status import (
    VideoStatus
)
from youtube_pydantic_models.models.part.statistics import (
    VideoStatistics
)
from youtube_pydantic_models.models.part.player import (
    VideoPlayer
)
from youtube_pydantic_models.models.part.topic_details import (
    VideoTopicDetails
)
from youtube_pydantic_models.models.part.recording_details import (
    BaseRecordingDetails
)
from youtube_pydantic_models.models.part.file_details import (
    BaseFileDetails
)
from youtube_pydantic_models.models.part.processing_details import (
    BaseProcessingDetails
)
from youtube_pydantic_models.models.part.suggestions import (
    BaseSuggestions
)
from youtube_pydantic_models.models.part.live_streaming_details import (
    BaseLiveStreamingDetails
)
from youtube_pydantic_models.models.subpart.localization import (
    Localization
)


class YoutubeVideoResource(YoutubeBaseResource):
    model_config = get_base_model_config()

    id: str | None = Field(default=None)
    snippet: VideoSnippet | None = Field(default=None)
    content_details: VideoContentDetails | None = Field(default=None)
    status: VideoStatus | None = Field(default=None)
    statistics: VideoStatistics | None = Field(default=None)
    player: VideoPlayer | None = Field(default=None)
    topic_details: VideoTopicDetails | None = Field(default=None)
    recording_details: BaseRecordingDetails | None = Field(default=None)
    file_details : BaseFileDetails | None = Field(default=None)
    processing_details: BaseProcessingDetails | None = Field(default=None)
    suggestions: BaseSuggestions | None = Field(default=None)
    live_streaming_details: BaseLiveStreamingDetails | None = Field(default=None)
    localizations: dict[str, Localization] | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class ContentRating(BaseModel):
    model_config = get_base_model_config()

    acb_rating: str | None = Field(default=None)
    agcom_rating: str | None = Field(default=None)
    anatel_rating: str | None = Field(default=None)
    bbfc_rating: str | None = Field(default=None)
    bfvc_rating: str | None = Field(default=None)
    bmukk_rating: str | None = Field(default=None)
    catv_rating: str | None = Field(default=None)
    catvfr_rating: str | None = Field(default=None)
    cbfc_rating: str | None = Field(default=None)
    ccc_rating: str | None = Field(default=None)
    cce_rating: str | None = Field(default=None)
    chfilm_rating: str | None = Field(default=None)
    chvrs_rating: str | None = Field(default=None)
    cicf_rating: str | None = Field(default=None)
    cna_rating: str | None = Field(default=None)
    cnc_rating: str | None = Field(default=None)
    csa_rating: str | None = Field(default=None)
    cscf_rating: str | None = Field(default=None)
    czfilm_rating: str | None = Field(default=None)
    djctq_rating: str | None = Field(default=None)
    djctq_rating_reasons: list[str] | None = Field(default=None)
    ecbmct_rating: str | None = Field(default=None)
    eefilm_rating: str | None = Field(default=None)
    egfilm_rating: str | None = Field(default=None)
    eirin_rating: str | None = Field(default=None)
    fcbm_rating: str | None = Field(default=None)
    fco_rating: str | None = Field(default=None)
    fmoc_rating: str | None = Field(default=None)
    fpb_rating: str | None = Field(default=None)
    fpb_rating_reasons: list[str] | None = Field(default=None)
    fsk_rating: str | None = Field(default=None)
    grfilm_rating: str | None = Field(default=None)
    icaa_rating: str | None = Field(default=None)
    ifco_rating: str | None = Field(default=None)
    ilfilm_rating: str | None = Field(default=None)
    incaa_rating: str | None = Field(default=None)
    kfcb_rating: str | None = Field(default=None)
    kijkwijzer_rating: str | None = Field(default=None)
    kmrb_rating: str | None = Field(default=None)
    lsf_rating: str | None = Field(default=None)
    mccaa_rating: str | None = Field(default=None)
    mccyp_rating: str | None = Field(default=None)
    mcst_rating: str | None = Field(default=None)
    mda_rating: str | None = Field(default=None)
    medietilsynet_rating: str | None = Field(default=None)
    meku_rating: str | None = Field(default=None)
    mibac_rating: str | None = Field(default=None)
    moc_rating: str | None = Field(default=None)
    moctw_rating: str | None = Field(default=None)
    mpaa_rating: str | None = Field(default=None)
    mpaat_rating: str | None = Field(default=None)
    mtrcb_rating: str | None = Field(default=None)
    nbc_rating: str | None = Field(default=None)
    nbcpl_rating: str | None = Field(default=None)
    nfrc_rating: str | None = Field(default=None)
    nfvcb_rating: str | None = Field(default=None)
    nkclv_rating: str | None = Field(default=None)
    oflc_rating: str | None = Field(default=None)
    pefilm_rating: str | None = Field(default=None)
    rcnof_rating: str | None = Field(default=None)
    resorteviolencia_rating: str | None = Field(default=None)
    rtc_rating: str | None = Field(default=None)
    rte_rating: str | None = Field(default=None)
    russia_rating: str | None = Field(default=None)
    skfilm_rating: str | None = Field(default=None)
    smais_rating: str | None = Field(default=None)
    smsa_rating: str | None = Field(default=None)
    tvpg_rating: str | None = Field(default=None)
    yt_rating: str | None = Field(default=None)



from youtube_pydantic_models.models.resources.base_resource import YoutubeBaseResource
from youtube_pydantic_models.models.resources.channel_resource import YoutubeChannelResource
from youtube_pydantic_models.models.resources.playlist_resource import YoutubePlaylistResource
from youtube_pydantic_models.models.resources.video_resource import YoutubeVideoResource
from youtube_pydantic_models.models.resources.search_resource import YoutubeSearchResource
from youtube_pydantic_models.models.youtube_client import YoutubeClient


# Python
from datetime import datetime
# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.subpart.thumbnail import (
    Thumbnail
)
from youtube_pydantic_models.models.subpart.localized import (
    Localized
)


class BaseSnippet(BaseModel):
    model_config = get_base_model_config()

    published_at: datetime | str | None = Field(default=None)
    title: str | None = Field(default=None)
    description: str | None = Field(default=None)
    thumbnails: dict[str, Thumbnail] | None = Field(default=None)


class SearchSnippet(BaseSnippet):
    model_config = get_base_model_config()

    channel_id: str | None = Field(default=None)
    channel_title: str | None = Field(default=None)
    live_broadcast_content: str | None = Field(default=None)
    publish_time: datetime | str | None = Field(default=None)


class ChannelSnippet(BaseSnippet):
    model_config = get_base_model_config()

    custom_url: str | None = Field(default=None)
    default_language: str | None = Field(default=None)
    localized: Localized | None = Field(default=None)
    country: str | None = Field(default=None)


class PlaylistSnippet(BaseSnippet):
    model_config = get_base_model_config()
    
    channel_id: str | None = Field(default=None)
    channel_title: str | None = Field(default=None)
    default_language: str | None = Field(default=None)
    localized: Localized | None = Field(default=None)


class VideoSnippet(PlaylistSnippet):
    model_config = get_base_model_config()

    tags: list[str] | None = Field(default=None)
    category_id: str | None = Field(default=None)
    live_broadcast_content: str | None = Field(default=None)
    default_audio_language: str | None = Field(default=None)



# Pydantic
from pydantic import Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.resources.base_resource import (
    YoutubeBaseResource
)
from youtube_pydantic_models.models.part.snippet import (
    PlaylistSnippet
)
from youtube_pydantic_models.models.part.status import (
    BaseStatus
)
from youtube_pydantic_models.models.part.content_details import (
    PlaylistContentDetails
)
from youtube_pydantic_models.models.part.player import (
    BasePlayer
)
from youtube_pydantic_models.models.subpart.localization import (
    Localization
)


class YoutubePlaylistResource(YoutubeBaseResource):
    model_config = get_base_model_config()

    id: str | None = Field(default=None)
    snippet: PlaylistSnippet | None = Field(default=None)
    status: BaseStatus | None = Field(default=None)
    content_details: PlaylistContentDetails | None = Field(default=None)
    player: BasePlayer | None = Field(default=None)
    localizations: dict[str, Localization] | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class SharedContentStream(BaseModel):
    model_config = get_base_model_config()

    channel_count: int | None = Field(default=None)
    codec: str | None = Field(default=None)
    bitrate_bps: int | None = Field(default=None)
    vendor: str | None = Field(default=None)


class AudioStream(SharedContentStream):
    model_config = get_base_model_config()

    channel_count: int | None = Field(default=None)


class VideoStream(SharedContentStream):
    model_config = get_base_model_config()

    width_pixels: int | None = Field(default=None)
    height_pixels: int | None = Field(default=None)
    frame_rate_fps: float | None = Field(default=None)
    aspect_ratio: float | None = Field(default=None)
    rotation: str | None = Field(default=None)



# Python
from datetime import datetime
# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class BaseLiveStreamingDetails(BaseModel):
    model_config = get_base_model_config()

    actual_start_time: datetime | str | None = Field(default=None)
    actual_end_time: datetime | str | None = Field(default=None)
    scheduled_start_time: datetime | str | None = Field(default=None)
    scheduled_end_time: datetime | str | None = Field(default=None)
    concurrent_viewers: int | None = Field(default=None)
    active_live_chat_id: str | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.subpart.related_playlists import (
    RelatedPlaylists
)
from youtube_pydantic_models.models.subpart.region_restriction import (
    RegionRestriction
)
from youtube_pydantic_models.models.subpart.content_rating import (
    ContentRating
)


class ChannelContentDetails(BaseModel):
    model_config = get_base_model_config()

    related_playlists: RelatedPlaylists | None = Field(default=None)


class PlaylistContentDetails(BaseModel):
    model_config = get_base_model_config()

    item_count: int | None = Field(default=None)


class VideoContentDetails(BaseModel):
    model_config = get_base_model_config()

    duration: str | None = Field(default=None)
    dimension: str | None = Field(default=None)
    definition: str | None = Field(default=None)
    caption: str | None = Field(default=None)
    licensed_content: bool | None = Field(default=None)
    region_restriction: RegionRestriction | None = Field(default=None)
    content_rating: ContentRating | None = Field(default=None)
    projection: str | None = Field(default=None)
    has_custom_thumbnail: bool | None = Field(default=None)



# Pydantic
from pydantic import Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.resources.base_resource import (
    YoutubeBaseResource
)
from youtube_pydantic_models.models.part.id import (
    SearchId
)
from youtube_pydantic_models.models.part.snippet import (
    SearchSnippet
)


class YoutubeSearchResource(YoutubeBaseResource):
    model_config = get_base_model_config()

    id: SearchId | None = Field(default=None)
    snippet: SearchSnippet | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class BaseAuditDetails(BaseModel):
    model_config = get_base_model_config()

    overall_good_standing: bool | None = Field(default=None)
    community_guide_lines_good_standing: bool | None = Field(default=None)
    copyright_strikes_good_standing: bool | None = Field(default=None)
    content_id_claims_good_standing: bool | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.subpart.processing_progress import (
    ProcessingProgress
)


class BaseProcessingDetails(BaseModel):
    model_config = get_base_model_config()

    processing_status: str | None = Field(default=None)
    processing_progress: ProcessingProgress | None = Field(default=None)
    processing_failure_reason: str | None = Field(default=None)
    file_details_availability: str | None = Field(default=None)
    processing_issues_availability: str | None = Field(default=None)
    tag_suggestions_availability: str | None = Field(default=None)
    editor_suggestions_availability: str | None = Field(default=None)
    thumbnails_availability: str | None = Field(default=None)



# Python
from datetime import datetime
# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class BaseContentOwnerDetails(BaseModel):
    model_config = get_base_model_config()

    content_owner: str | None = Field(default=None)
    time_linked: datetime | str | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.subpart.channel import (
    Channel
)
from youtube_pydantic_models.models.subpart.image import (
    Image
)
from youtube_pydantic_models.models.subpart.watch import (
    Watch
)


class BaseBrandingSettings(BaseModel):
    model_config = get_base_model_config()

    channel: Channel | None = Field(default=None)
    image: Image | None = Field(default=None)
    watch: Watch | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.subpart.audio_video_stream import (
    AudioStream,
    VideoStream
)


class BaseFileDetails(BaseModel):
    model_config = get_base_model_config()

    file_name: str | None = Field(default=None)
    file_size: int | None = Field(default=None)
    file_type: str | None = Field(default=None)
    container: str | None = Field(default=None)
    video_streams: list[VideoStream] | None = Field(default=None)
    audio_streams: list[AudioStream] | None = Field(default=None)
    duration_ms: int | None = Field(default=None)
    bitrate_bps: int | None = Field(default=None)
    creation_time: str | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.models.subpart.tag_suggestion import (
    TagSuggestions
)


class BaseSuggestions(BaseModel):
    model_config = get_base_model_config()

    processing_errors: list[str] | None = Field(default=None)
    processing_warnings: list[str] | None = Field(default=None)
    processing_hints: list[str] | None = Field(default=None)
    tag_suggestions: TagSuggestions | None = Field(default=None)
    editor_suggestions: list[str] | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class ProcessingProgress(BaseModel):
    model_config = get_base_model_config()

    parts_total: int | None = Field(default=None)
    parts_processed: int | None = Field(default=None)
    time_left_ms: int | None = Field(default=None)



# Pydantic
from pydantic import BaseModel, Field
# models
from youtube_pydantic_models.models.base_model_config import (
    get_base_model_config
)


class BaseStatistics(BaseModel):
    model_config = get_base_model_config()

    view_count: str | None = Field(default=None)


class ChannelStatistics(BaseStatistics):
    model_config = get_base_model_config()

    subscriber_count: str | None = Field(default=None)
    hidden_subscriber_count: bool | None = Field(default=None)
    video_count: str | None = Field(default=None)


class VideoStatistics(BaseStatistics):
    model_config = get_base_model_config()

    like_count: str | None = Field(default=None)
    dislike_count: str | None = Field(default=None)
    favorite_count: str | None = Field(default=None)
    comment_count: str | None = Field(default=None)
